//---------------------------------------------------------------------------
#include "WeibullClimateSurvival.h"
#include "TreePopulation.h"
#include "SimManager.h"
#include "ParsingFunctions.h"
#include "Plot.h"
#include "Allometry.h"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
// Constructor
//////////////////////////////////////////////////////////////////////////////
clWeibullClimateSurvival::clWeibullClimateSurvival( clSimManager * p_oSimManager ) :
  clWorkerBase( p_oSimManager ), clBehaviorBase( p_oSimManager ),
  clMortalityBase( p_oSimManager )
{
  try
  {
    //Set namestring
    strcpy( m_cNameString, "WeibullClimatemortshell" );

    //Null out our pointers
    mp_fCompC = NULL;
    mp_fCompD = NULL;
    mp_fGamma = NULL;
    mp_fMaxCrowdingRadius = NULL;
    mp_fMinimumNeighborDBH = NULL;
    mp_fSizeX0 = NULL;
    mp_fSizeXb = NULL;
    mp_fSizeMinDBH = NULL;
    mp_fPrecipA = NULL;
    mp_fPrecipB = NULL;
    mp_fPrecipC = NULL;
    mp_fTempA = NULL;
    mp_fTempB = NULL;
    mp_fTempC = NULL;
    mp_fMaxRG = NULL;
    mp_iIndexes = NULL;
    mp_fPrecipEffect = NULL;
    mp_fTempEffect = NULL;

    //Version 1
    m_fVersionNumber = 1.0;
    m_fMinimumVersionNumber = 1.0;
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clWeibullClimateSurvival::clWeibullClimateSurvival" );
    throw( stcErr );
  }
}


////////////////////////////////////////////////////////////////////////////
// Destructor
////////////////////////////////////////////////////////////////////////////
clWeibullClimateSurvival::~clWeibullClimateSurvival()
{
  int i, iFirstIndex = 2;
  if ( mp_fCompC ) for (i=0; i<iFirstIndex; i++) delete[] mp_fCompC[i];
  if ( mp_fCompD ) for (i=0; i<iFirstIndex; i++) delete[] mp_fCompD[i];
  if ( mp_fGamma ) for (i=0; i<iFirstIndex; i++) delete[] mp_fGamma[i];
  if ( mp_fPrecipA ) for (i=0; i<iFirstIndex; i++) delete[] mp_fPrecipA[i];
  if ( mp_fPrecipB ) for (i=0; i<iFirstIndex; i++) delete[] mp_fPrecipB[i];
  if ( mp_fPrecipC ) for (i=0; i<iFirstIndex; i++) delete[] mp_fPrecipC[i];
  if ( mp_fTempA ) for (i=0; i<iFirstIndex; i++) delete[] mp_fTempA[i];
  if ( mp_fTempB ) for (i=0; i<iFirstIndex; i++) delete[] mp_fTempB[i];
  if ( mp_fTempC ) for (i=0; i<iFirstIndex; i++) delete[] mp_fTempC[i];
  if ( mp_fMaxRG ) for (i=0; i<iFirstIndex; i++) delete[] mp_fMaxRG[i];
  if ( mp_fSizeX0 ) for (i=0; i<iFirstIndex; i++) delete[] mp_fSizeX0[i];
  if ( mp_fSizeXb ) for (i=0; i<iFirstIndex; i++) delete[] mp_fSizeXb[i];
  if ( mp_fPrecipEffect ) for (i=0; i<iFirstIndex; i++) delete[] mp_fPrecipEffect[i];
  if ( mp_fTempEffect ) for (i=0; i<iFirstIndex; i++) delete[] mp_fTempEffect[i];

  delete[] mp_fCompC;
  delete[] mp_fCompD;
  delete[] mp_fGamma;
  delete[] mp_fMaxCrowdingRadius;
  delete[] mp_fMinimumNeighborDBH;
  delete[] mp_fSizeX0;
  delete[] mp_fSizeXb;
  delete[] mp_fSizeMinDBH;
  delete[] mp_fPrecipA;
  delete[] mp_fPrecipB;
  delete[] mp_fPrecipC;
  delete[] mp_fTempA;
  delete[] mp_fTempB;
  delete[] mp_fTempC;
  delete[] mp_fMaxRG;
  delete[] mp_iIndexes;
  delete[] mp_fPrecipEffect;
  delete[] mp_fTempEffect;
}


////////////////////////////////////////////////////////////////////////////
// ReadParameterFile()
////////////////////////////////////////////////////////////////////////////
void clWeibullClimateSurvival::ReadParameterFile( xercesc::DOMDocument * p_oDoc )
{
  try
  {
    clTreePopulation * p_oPop = ( clTreePopulation * ) mp_oSimManager->GetPopulationObject( "treepopulation" );
    DOMElement * p_oElement = p_oDoc->getDocumentElement();
    floatVal * p_fTempValues; //for getting species-specific values
    doubleVal *p_dTempValues;
    short int i; //loop counter

    m_fMinSaplingHeight = 50;

    m_iNumTotalSpecies = p_oPop->GetNumberOfSpecies();
    //Get the minimum sapling height
    for ( i = 0; i < m_iNumTotalSpecies; i++ )
    {
      if ( p_oPop->GetMaxSeedlingHeight( i ) < m_fMinSaplingHeight )
      {
        m_fMinSaplingHeight = p_oPop->GetMaxSeedlingHeight( i );
      }
    }

    //If any of the types is seedling, error out
    for ( i = 0; i < m_iNumSpeciesTypeCombos; i++ )
      if ( clTreePopulation::sapling != mp_whatSpeciesTypeCombos[i].iType
           && clTreePopulation::adult != mp_whatSpeciesTypeCombos[i].iType )
           {
             modelErr stcErr;
             stcErr.iErrorCode = BAD_DATA;
             strcpy( stcErr.cFunction, "clWeibullClimateSurvival::ReadParameterFile" );
             strcpy( stcErr.cMoreInfo, "This behavior can only be applied to saplings and adults." );
             throw( stcErr );
      }

    //Make the list of indexes
    mp_iIndexes = new short int[m_iNumTotalSpecies];
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_iIndexes[mp_iWhatSpecies[i]] = i;

    //This alone is sized total number of species
    mp_fMinimumNeighborDBH = new float[m_iNumTotalSpecies];

    //The rest are sized number of species to which this behavior applies
    mp_fCompC = new float*[2];
    mp_fCompD = new float*[2];
    mp_fGamma = new float*[2];
    mp_fPrecipA = new float*[2];
    mp_fPrecipB = new float*[2];
    mp_fPrecipC = new float*[2];
    mp_fTempA = new float*[2];
    mp_fTempB = new float*[2];
    mp_fTempC = new float*[2];
    mp_fMaxRG = new float*[2];
    mp_fSizeX0 = new double*[2];
    mp_fSizeXb = new float*[2];
    for ( i = 0; i < 2; i++ ) {
      mp_fCompC[i] = new float[m_iNumBehaviorSpecies];
      mp_fCompD[i] = new float[m_iNumBehaviorSpecies];
      mp_fGamma[i] = new float[m_iNumBehaviorSpecies];
      mp_fPrecipA[i] = new float[m_iNumBehaviorSpecies];
      mp_fPrecipB[i] = new float[m_iNumBehaviorSpecies];
      mp_fPrecipC[i] = new float[m_iNumBehaviorSpecies];
      mp_fTempA[i] = new float[m_iNumBehaviorSpecies];
      mp_fTempB[i] = new float[m_iNumBehaviorSpecies];
      mp_fTempC[i] = new float[m_iNumBehaviorSpecies];
      mp_fMaxRG[i] = new float[m_iNumBehaviorSpecies];
      mp_fSizeX0[i] = new double [m_iNumBehaviorSpecies];
      mp_fSizeXb[i] = new float[m_iNumBehaviorSpecies];
    }

    mp_fMaxCrowdingRadius = new float[m_iNumBehaviorSpecies];
    mp_fSizeMinDBH = new float[m_iNumBehaviorSpecies];

    //Set up our floatVal array that will extract values only for the species
    //assigned to this behavior
    p_fTempValues = new floatVal[m_iNumBehaviorSpecies];
    p_dTempValues = new doubleVal[m_iNumBehaviorSpecies];
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ ) {
      p_fTempValues[i].code = mp_iWhatSpecies[i];
      p_dTempValues[i].code = mp_iWhatSpecies[i];
    }

    //Fill the variables

    //Adult maximum potential survival
    FillSpeciesSpecificValue( p_oElement, "mo_weibClimAdultMaxSurv",
        "mo_wcamsVal", p_fTempValues,
         m_iNumBehaviorSpecies, p_oPop, true );
    //Transfer to the appropriate array buckets
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fMaxRG[1][i] = p_fTempValues[i].val;

    //Juvenile maximum potential survival
    FillSpeciesSpecificValue( p_oElement, "mo_weibClimJuvMaxSurv",
        "mo_wcjmsVal", p_fTempValues,
         m_iNumBehaviorSpecies, p_oPop, true );
    //Transfer to the appropriate array buckets
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fMaxRG[0][i] = p_fTempValues[i].val;

    //Max crowding radius
    FillSpeciesSpecificValue( p_oElement, "mo_weibClimMaxNeighRad", "mo_wcmnrVal", p_fTempValues,
         m_iNumBehaviorSpecies, p_oPop, true );
    //Transfer to the appropriate array buckets
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fMaxCrowdingRadius[i] = p_fTempValues[i].val;

    //Adult size sensitivity to crowding parameter (gamma)
    FillSpeciesSpecificValue( p_oElement, "mo_weibClimAdultCompEffGamma",
        "mo_wcacegVal", p_fTempValues, m_iNumBehaviorSpecies, p_oPop, true );
    //Transfer to the appropriate array buckets
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fGamma[1][i] = p_fTempValues[i].val;

    //Juvenile size sensitivity to crowding parameter (gamma)
    FillSpeciesSpecificValue( p_oElement, "mo_weibClimJuvCompEffGamma",
        "mo_wcjcegVal", p_fTempValues, m_iNumBehaviorSpecies, p_oPop, true );
    //Transfer to the appropriate array buckets
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fGamma[0][i] = p_fTempValues[i].val;

    //Minimum neighbor DBH
    FillSpeciesSpecificValue( p_oElement, "mo_weibClimMinNeighDBH",
        "mo_wcmndVal", mp_fMinimumNeighborDBH, p_oPop, true );

    //Adult C
    FillSpeciesSpecificValue( p_oElement, "mo_weibClimAdultCompEffC",
        "mo_wcacecVal", p_fTempValues, m_iNumBehaviorSpecies, p_oPop, true );
    //Transfer to the appropriate array buckets
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fCompC[1][i] = p_fTempValues[i].val;

    //Juvenile C
    FillSpeciesSpecificValue( p_oElement, "mo_weibClimJuvCompEffC",
        "mo_wcjcecVal", p_fTempValues, m_iNumBehaviorSpecies, p_oPop, true );
    //Transfer to the appropriate array buckets
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fCompC[0][i] = p_fTempValues[i].val;

    //Adult D
    FillSpeciesSpecificValue( p_oElement, "mo_weibClimAdultCompEffD",
        "mo_wcacedVal", p_fTempValues, m_iNumBehaviorSpecies, p_oPop, true );
    //Transfer to the appropriate array buckets
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fCompD[1][i] = p_fTempValues[i].val;

    //Juvenile D
    FillSpeciesSpecificValue( p_oElement, "mo_weibClimJuvCompEffD",
        "mo_wcjcedVal", p_fTempValues, m_iNumBehaviorSpecies, p_oPop, true );
    //Transfer to the appropriate array buckets
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fCompD[0][i] = p_fTempValues[i].val;

    //Adult size effect mode (X0)
    FillSpeciesSpecificValue( p_oElement, "mo_weibClimAdultSizeEffX0",
        "mo_wcasex0Val", p_dTempValues, m_iNumBehaviorSpecies, p_oPop, true );
    //Transfer to the appropriate array buckets
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fSizeX0[1][i] = p_dTempValues[i].val;

    //Juvenile size effect mode (X0)
    FillSpeciesSpecificValue( p_oElement, "mo_weibClimJuvSizeEffX0",
        "mo_wcjsex0Val", p_dTempValues, m_iNumBehaviorSpecies, p_oPop, true );
    //Transfer to the appropriate array buckets
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fSizeX0[0][i] = p_dTempValues[i].val;

    //Adult size effect variance (Xb)
    FillSpeciesSpecificValue( p_oElement, "mo_weibClimAdultSizeEffXb",
        "mo_wcasexbVal", p_fTempValues, m_iNumBehaviorSpecies, p_oPop, true );
    //Transfer to the appropriate array buckets
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fSizeXb[1][i] = p_fTempValues[i].val;

    //Juvenile size effect variance (Xb)
    FillSpeciesSpecificValue( p_oElement, "mo_weibClimJuvSizeEffXb",
        "mo_wcjsexbVal", p_fTempValues, m_iNumBehaviorSpecies, p_oPop, true );
    //Transfer to the appropriate array buckets
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fSizeXb[0][i] = p_fTempValues[i].val;

    //Size effect minimum DBH
    FillSpeciesSpecificValue( p_oElement, "mo_weibClimSizeEffMinDBH",
        "mo_wcsemdVal", p_fTempValues, m_iNumBehaviorSpecies, p_oPop, true );
    //Transfer to the appropriate array buckets
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fSizeMinDBH[i] = p_fTempValues[i].val;

    //Adult temperature effect a
    FillSpeciesSpecificValue( p_oElement, "mo_weibClimAdultTempEffA",
        "mo_wcateaVal", p_fTempValues, m_iNumBehaviorSpecies, p_oPop, true );
    //Transfer to the appropriate array buckets
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fTempA[1][i] = p_fTempValues[i].val;

    //Juvenile temperature effect a
    FillSpeciesSpecificValue( p_oElement, "mo_weibClimJuvTempEffA",
        "mo_wcjteaVal", p_fTempValues, m_iNumBehaviorSpecies, p_oPop, true );
    //Transfer to the appropriate array buckets
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fTempA[0][i] = p_fTempValues[i].val;

    //Adult temperature effect b
    FillSpeciesSpecificValue( p_oElement, "mo_weibClimAdultTempEffB",
        "mo_wcatebVal", p_fTempValues, m_iNumBehaviorSpecies, p_oPop, true );
    //Transfer to the appropriate array buckets
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fTempB[1][i] = p_fTempValues[i].val;

    //Juvenile temperature effect b
    FillSpeciesSpecificValue( p_oElement, "mo_weibClimJuvTempEffB",
        "mo_wcjtebVal", p_fTempValues, m_iNumBehaviorSpecies, p_oPop, true );
    //Transfer to the appropriate array buckets
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fTempB[0][i] = p_fTempValues[i].val;

    //Adult temperature effect c
    FillSpeciesSpecificValue( p_oElement, "mo_weibClimAdultTempEffC",
        "mo_wcatecVal", p_fTempValues, m_iNumBehaviorSpecies, p_oPop, true );
    //Transfer to the appropriate array buckets
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fTempC[1][i] = p_fTempValues[i].val;

    //Juvenile temperature effect c
    FillSpeciesSpecificValue( p_oElement, "mo_weibClimJuvTempEffC",
        "mo_wcjtecVal", p_fTempValues, m_iNumBehaviorSpecies, p_oPop, true );
    //Transfer to the appropriate array buckets
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fTempC[0][i] = p_fTempValues[i].val;

    //Adult precipitation effect a
    FillSpeciesSpecificValue( p_oElement, "mo_weibClimAdultPrecipEffA",
        "mo_wcapeaVal", p_fTempValues, m_iNumBehaviorSpecies, p_oPop, true );
    //Transfer to the appropriate array buckets
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fPrecipA[1][i] = p_fTempValues[i].val;

    //Juvenile precipitation effect a
    FillSpeciesSpecificValue( p_oElement, "mo_weibClimJuvPrecipEffA",
        "mo_wcjpeaVal", p_fTempValues, m_iNumBehaviorSpecies, p_oPop, true );
    //Transfer to the appropriate array buckets
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fPrecipA[0][i] = p_fTempValues[i].val;

    //Adult precipitation effect b
    FillSpeciesSpecificValue( p_oElement, "mo_weibClimAdultPrecipEffB",
        "mo_wcapebVal", p_fTempValues, m_iNumBehaviorSpecies, p_oPop, true );
    //Transfer to the appropriate array buckets
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fPrecipB[1][i] = p_fTempValues[i].val;

    //Juvenile precipitation effect b
    FillSpeciesSpecificValue( p_oElement, "mo_weibClimJuvPrecipEffB",
        "mo_wcjpebVal", p_fTempValues, m_iNumBehaviorSpecies, p_oPop, true );
    //Transfer to the appropriate array buckets
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fPrecipB[0][i] = p_fTempValues[i].val;

    //Adult precipitation effect c
    FillSpeciesSpecificValue( p_oElement, "mo_weibClimAdultPrecipEffC",
        "mo_wcapecVal", p_fTempValues, m_iNumBehaviorSpecies, p_oPop, true );
    //Transfer to the appropriate array buckets
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fPrecipC[1][i] = p_fTempValues[i].val;

    //Juvenile precipitation effect c
    FillSpeciesSpecificValue( p_oElement, "mo_weibClimJuvPrecipEffC",
        "mo_wcjpecVal", p_fTempValues, m_iNumBehaviorSpecies, p_oPop, true );
    //Transfer to the appropriate array buckets
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fPrecipC[0][i] = p_fTempValues[i].val;

    delete[] p_fTempValues;
    delete[] p_dTempValues;
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clWeibullClimateSurvival::ReadParameterFile" );
    throw( stcErr );
  }
}

////////////////////////////////////////////////////////////////////////////
// ValidateData
////////////////////////////////////////////////////////////////////////////
void clWeibullClimateSurvival::ValidateData()
{
  try
  {

    int i;
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
    {

      //Make sure that the max radius of neighbor effects is > 0
      if ( mp_fMaxCrowdingRadius[i] <= 0 )
      {
        modelErr stcErr;
        strcpy( stcErr.cMoreInfo, "All values for max crowding radius must be greater than 0." );
        throw( stcErr );
      }

      //Make sure that the maximum survival for each species is between 0 and 1
      if ( mp_fMaxRG[1][i] < 0 || mp_fMaxRG[1][i] > 1 ||
           mp_fMaxRG[0][i] < 0 || mp_fMaxRG[0][i] > 1)
      {
        modelErr stcErr;
        strcpy( stcErr.cMoreInfo, "All values for max potential survival must be between 0 and 1." );
        throw( stcErr );
      }

      //Make sure that the size effect mode is not 0
      if ( 0 >= mp_fSizeX0[0][i] || 0 >= mp_fSizeX0[1][i] )
      {
        modelErr stcErr;
        strcpy( stcErr.cMoreInfo, "Size effect mode values must be greater than 0." );
        throw( stcErr );
      }

      //Make sure that the size effect variance is not 0
      if ( 0 == mp_fSizeXb[0][i] || 0 == mp_fSizeXb[1][i] )
      {
        modelErr stcErr;
        strcpy( stcErr.cMoreInfo, "Size effect variance values cannot be 0." );
        throw( stcErr );
      }

      //Make sure that the adult temp a is not 0
      if ( 0 == mp_fTempA[1][i] )
      {
        modelErr stcErr;
        strcpy( stcErr.cMoreInfo, "Temperature effect adult A values cannot be 0." );
        throw( stcErr );
      }

      //Make sure that the adult precip a is not 0
      if ( 0 == mp_fPrecipA[1][i] )
      {
        modelErr stcErr;
        strcpy( stcErr.cMoreInfo, "Precipitation effect adult A values cannot be 0." );
        throw( stcErr );
      }

      //Make sure that the juvenile temp a is not 0
      if ( 0 == mp_fTempA[0][i] )
      {
        modelErr stcErr;
        strcpy( stcErr.cMoreInfo, "Temperature effect juvenile A values cannot be 0." );
        throw( stcErr );
      }

      //Make sure that the juvenile precip a is not 0
      if ( 0 == mp_fPrecipA[0][i] )
      {
        modelErr stcErr;
        strcpy( stcErr.cMoreInfo, "Precipitation effect juvenile A values cannot be 0." );
        throw( stcErr );
      }
    }

    for ( i = 0; i < m_iNumTotalSpecies; i++ )
    {
      //Make sure that the minimum neighbor DBH is not negative
      if ( 0 > mp_fMinimumNeighborDBH[i] )
      {
        modelErr stcErr;
        strcpy( stcErr.cMoreInfo, "Minimum neighbor DBH for NCI cannot be less than 0." );
        throw( stcErr );
      }
    }
  }
  catch ( modelErr & err )
  {
    strcpy( err.cFunction, "clWeibullClimateSurvival::ValidateData" );
    err.iErrorCode = BAD_DATA;
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clWeibullClimateSurvival::ValidateData" );
    throw( stcErr );
  }
}

////////////////////////////////////////////////////////////////////////////
// DoShellSetup()
////////////////////////////////////////////////////////////////////////////
void clWeibullClimateSurvival::DoShellSetup( xercesc::DOMDocument * p_oDoc )
{
  ReadParameterFile( p_oDoc );
  ValidateData();

  mp_fTempEffect = new float*[2];
  mp_fPrecipEffect = new float*[2];
  for ( int i = 0; i < 2; i++ ) {
    mp_fTempEffect[i] = new float[m_iNumBehaviorSpecies];
    mp_fPrecipEffect[i] = new float[m_iNumBehaviorSpecies];
  }
}

////////////////////////////////////////////////////////////////////////////
// DoMort
////////////////////////////////////////////////////////////////////////////
deadCode clWeibullClimateSurvival::DoMort( clTree * p_oTree, const float & fDbh, const short int & iSpecies )
{

    float fCrowdingEffect, //tree's crowding effect
          fNumberYearsPerTimestep = mp_oSimManager->GetNumberOfYearsPerTimestep(),
          fSizeEffect, //tree's size effect
          fSEDbh,   //DBH for the purposes of calculating the size effect
          fSurvivalProb; //Tree's annual survival probability
    int iNumBiggerNeighbors;
    short int iType, //type and species of a tree
              iLHInd, iSpInd;
    bool bIsDead;

    iType = p_oTree->GetType();

    //Get a number of neighbors with a DBH greater than the target
    iNumBiggerNeighbors = GetNumLargerNeighbors(p_oTree);

    //Get the appropriate array index for the life history stage
    if ( clTreePopulation::sapling == iType)
      iLHInd = 0;
    else iLHInd = 1;

    iSpInd = mp_iIndexes[iSpecies];


    fCrowdingEffect = exp(-mp_fCompC[iLHInd][iSpInd] *
                 pow(fDbh, mp_fGamma[iLHInd][iSpInd]) *
                 pow(iNumBiggerNeighbors, mp_fCompD[iLHInd][iSpInd]));
    //Make sure it's between 0 and 1
    if ( fCrowdingEffect < 0 ) fCrowdingEffect = 0;
    if ( fCrowdingEffect > 1 ) fCrowdingEffect = 1;

    //Get the tree's size effect
    if (fDbh < mp_fSizeMinDBH[iSpInd])
      fSEDbh = mp_fSizeMinDBH[iSpInd];
    else
      fSEDbh = fDbh;
    fSizeEffect = exp( -0.5 * pow(
        log( fSEDbh / mp_fSizeX0[iLHInd][iSpInd] ) /
             mp_fSizeXb[iLHInd][iSpInd], 2 ) );

    //Make sure it's bounded between 0 and 1
    if ( fSizeEffect < 0 ) fSizeEffect = 0;
    if ( fSizeEffect > 1 ) fSizeEffect = 1;

    //Calculate annual survival probability
    fSurvivalProb = mp_fMaxRG[iLHInd][iSpInd] *
                        fSizeEffect *
                        fCrowdingEffect *
                        mp_fPrecipEffect[iLHInd][iSpInd] *
                        mp_fTempEffect[iLHInd][iSpInd];

    fSurvivalProb = pow( fSurvivalProb, fNumberYearsPerTimestep );
    bIsDead = clModelMath::GetRand() >= fSurvivalProb;

    if (bIsDead) return natural;
    else return notdead;

}



////////////////////////////////////////////////////////////////////////////
// GetNumLargerNeighbors
////////////////////////////////////////////////////////////////////////////
int clWeibullClimateSurvival::GetNumLargerNeighbors(clTree *p_oTarget)
{
  clTreePopulation * p_oPop = ( clTreePopulation * ) mp_oSimManager->GetPopulationObject( "treepopulation" );
  clTreeSearch * p_oAllNeighbors; //neighborhood trees within crowding radius
  clTree * p_oNeighbor; //competing neighbor
  char cQuery[75]; //format search strings into this
  float fNeighDbh, //neighbor's dbh
        fTargetDbh, //target's dbh
        fTargetX, fTargetY; //holders for the target tree's X and Y location
  int iIsDead, //whether a neighbor is dead
      iNumNeighbors = 0;
  short int iNeighSpecies, iNeighType, //species and type for neighbor
       iTargetSpecies = p_oTarget->GetSpecies(), //target tree's species
       iTargetType = p_oTarget->GetType(), //target tree's type
       iDeadCode; //neighbor's dead code

    //Get the target tree's DBH
    p_oTarget->GetValue( p_oPop->GetDbhCode( iTargetSpecies, iTargetType ), & fTargetDbh );

    //Format the query to get all competing neighbors
    p_oTarget->GetValue( p_oPop->GetXCode( iTargetSpecies, iTargetType ), & fTargetX );
    p_oTarget->GetValue( p_oPop->GetYCode( iTargetSpecies, iTargetType ), & fTargetY );

    //Get all trees taller than seedlings within the max crowding radius -
    //seedlings don't compete
    sprintf( cQuery, "%s%f%s%f%s%f%s%f", "distance=", mp_fMaxCrowdingRadius[mp_iIndexes[iTargetSpecies]], "FROM x=", fTargetX,
         "y=", fTargetY, "::height=", m_fMinSaplingHeight );
    p_oAllNeighbors = p_oPop->Find( cQuery );

    //Loop through and find all the bigger neighbors
    p_oNeighbor = p_oAllNeighbors->NextTree();

    while ( p_oNeighbor ) {

      iNeighSpecies = p_oNeighbor->GetSpecies();
      iNeighType = p_oNeighbor->GetType();

      if ( clTreePopulation::seedling != iNeighType &&
           clTreePopulation::snag != iNeighType) {

        //Get the neighbor's dbh
        p_oNeighbor->GetValue( p_oPop->GetDbhCode( iNeighSpecies, iNeighType ), & fNeighDbh );

        if ( fNeighDbh >= mp_fMinimumNeighborDBH[iNeighSpecies] &&
             fNeighDbh > fTargetDbh) {

          //Make sure the neighbor's not dead due to a disturbance event
          iDeadCode = p_oPop->GetIntDataCode( "dead", iNeighSpecies, iNeighType );
          if ( -1 != iDeadCode )  p_oNeighbor->GetValue( iDeadCode, & iIsDead );
          else iIsDead = notdead;

          if ( notdead == iIsDead || natural == iIsDead)
            iNumNeighbors++;
        }
      }

      p_oNeighbor = p_oAllNeighbors->NextTree();
    }
    return iNumNeighbors;
}

////////////////////////////////////////////////////////////////////////////
// PreMortCalcs
////////////////////////////////////////////////////////////////////////////
void clWeibullClimateSurvival::PreMortCalcs( clTreePopulation * p_oPop )
{
  clPlot *p_oPlot = mp_oSimManager->GetPlotObject();
  float fPlotTemp = p_oPlot->GetMeanAnnualTemp(), //Current plot temperature
        fPlotPrecip = p_oPlot->GetMeanAnnualPrecip(); //Current plot precipitation
  int i, j;

  for ( i = 0; i < 2; i++ ) {
    for ( j = 0; j < m_iNumBehaviorSpecies; j++ ) {
      mp_fPrecipEffect[i][j] = exp(-0.5*pow(fabs(fPlotPrecip - mp_fPrecipC[i][j])/mp_fPrecipA[i][j], mp_fPrecipB[i][j]));
      mp_fTempEffect[i][j] = exp(-0.5*pow(fabs(fPlotTemp - mp_fTempC[i][j])/mp_fTempA[i][j], mp_fTempB[i][j]));
    }
  }

}
